"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const diffutils_1 = __importDefault(require("../../../shared/diffutils"));
const path = __importStar(require("path"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_diff");
class Diff extends command_1.SfdxCommand {
    async run() {
        const diffFile = this.flags.difffile;
        let encoding = this.flags.encoding;
        const outputFolder = this.flags.output;
        const revisionfrom = this.flags.revisionfrom;
        const revisionto = this.flags.revisionto;
        if (!encoding || encoding === "") {
            encoding = "utf8";
        }
        if ((diffFile === undefined || diffFile === '') && (revisionfrom === undefined || revisionfrom === '')) {
            this.error('Provide either diffFile or revisionFrom parameters');
        }
        let diffUtils = new diffutils_1.default(revisionfrom, revisionto);
        /* PATH TO DIFF FILE */
        let diffFilePath = '';
        if (diffFile) {
            diffFilePath = path.join(process.cwd(), diffFile);
        }
        let diffOutput = await diffUtils.build(diffFilePath, encoding, outputFolder);
        if (!this.flags.json)
            this.ux.logJson(diffOutput);
        return diffOutput;
    }
}
Diff.description = messages.getMessage("commandDescription");
Diff.examples = [
    `$ sfdx sfpowerkit:project:diff --diffFile DiffFileName --encoding EncodingOfFile --output OutputFolder
    {
      "status": 0,
      "result": {
        "deleted": [],
        "addedEdited": [
          "scripts\\Alias.sh",
          "sfdx-project.json",
        ]
       }
      }`,
    `$ sfdx sfpowerkit:project:diff --revisionfrom revisionfrom --revisionto revisionto --output OutputFolder
   {
    "status": 0,
    "result": {
      "deleted": [],
      "addedEdited": [
        "scripts\\Alias.sh",
        "sfdx-project.json",
      ]
     }
    }
   `
];
Diff.flagsConfig = {
    difffile: command_1.flags.string({ char: 'f', description: messages.getMessage('diffFileDescription'), required: false }),
    encoding: command_1.flags.string({ char: 'e', description: messages.getMessage('encodingDescription'), required: false }),
    revisionfrom: command_1.flags.string({ char: 'r', description: messages.getMessage('revisionFromDescription'), required: false }),
    revisionto: command_1.flags.string({ char: 't', description: messages.getMessage('revisionToDescription'), required: false }),
    output: command_1.flags.string({ char: 'd', description: messages.getMessage('outputFolderDescription'), required: true })
};
Diff.requiresUsername = false;
Diff.requiresProject = true;
exports.default = Diff;
//# sourceMappingURL=diff.js.map