"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
const getPackageInfo_1 = require("../../../../shared/getPackageInfo");
var path = require('path');
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages('sfpowerkit', 'apextestsuite_convert');
class Convert extends command_1.SfdxCommand {
    async run() {
        rimraf.sync('temp_sfpowerkit');
        // Getting Project config
        const project = await core_1.SfdxProject.resolve();
        const projectJson = await project.retrieveSfdxProjectJson();
        //Retrieve the package
        let packageToBeUsed;
        if (this.flags.package)
            packageToBeUsed = getPackageInfo_1.getPackageInfo(projectJson, this.flags.package);
        else
            packageToBeUsed = getPackageInfo_1.getDefaultPackageInfo(projectJson);
        //Check for the apextestsuite in the path
        let apextestsuite_file_path = packageToBeUsed.path + `/main/default/testsuites/${this.flags.name}.testSuite-meta.xml`;
        if (this.flags.pathOverride)
            apextestsuite_file_path = packageToBeUsed.path + this.flags.pathOverride + `/testsuites/${this.flags.name}.testSuite-meta.xml`;
        if (fs_extra_1.default.existsSync(path.resolve(apextestsuite_file_path))) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            let apex_test_suite = await parseString(fs_extra_1.default.readFileSync(path.resolve(apextestsuite_file_path)));
            let apex_test_suite_as_string = JSON.stringify(apex_test_suite.ApexTestSuite.testClassName);
            apex_test_suite_as_string = apex_test_suite_as_string.replace(/("|')/g, "");
            apex_test_suite_as_string = apex_test_suite_as_string.slice(1, -1);
            apex_test_suite_as_string = '\"' + apex_test_suite_as_string.concat('\"');
            this.ux.log(apex_test_suite_as_string);
        }
        else {
            throw new core_1.SfdxError("Apex Test Suite not found");
        }
        return 0;
    }
}
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Convert.requiresProject = true;
Convert.description = messages.getMessage('commandDescription');
Convert.examples = [
    `$ sfdx  sfpowerkit:source:apextestsuite:convert -n MyApexTestSuite 
    "ABC2,ABC1Test"    
  `
];
Convert.flagsConfig = {
    name: command_1.flags.string({ required: true, char: 'n', description: messages.getMessage('nameFlagDescription') }),
    package: command_1.flags.string({ required: false, char: 'p', description: messages.getMessage('packageFlagDescription') }),
    pathoverride: command_1.flags.string({ required: false, char: 'o', description: messages.getMessage('pathFlagDescription') })
};
exports.default = Convert;
//# sourceMappingURL=convert.js.map