"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
class MetadataFiles {
    static getFullApiName(fileName) {
        let fullName = "";
        let metadateType = MetadataFiles.getNameOfTypes(fileName);
        let splitFilepath = fileName.split(path.sep);
        if (metadateType === 'CustomField' || metadateType === 'RecordType' || metadateType === 'ListView' || metadateType === 'ValidationRule' || metadateType === 'WebLink' || metadateType === 'CompactLayout' || metadateType === 'BusinessProcess') {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1].split('.')[0];
            fullName = objectName.concat('.' + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1].split('.')[0];
        }
        return fullName;
    }
    static getFullApiNameWithExtension(fileName) {
        let fullName = "";
        let metadateType = MetadataFiles.getNameOfTypes(fileName);
        let splitFilepath = fileName.split(path.sep);
        if (metadateType === 'CustomField' || metadateType === 'RecordType' || metadateType === 'ListView' || metadateType === 'ValidationRule' || metadateType === 'WebLink' || metadateType === 'CompactLayout' || metadateType === 'BusinessProcess') {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1];
            fullName = objectName.concat('.' + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1];
        }
        return fullName;
    }
    static isCustomMetadata(filepath, name) {
        let result = true;
        let splitFilepath = filepath.split(path.sep);
        let componentName = splitFilepath[splitFilepath.length - 1];
        componentName = componentName.substring(0, componentName.indexOf("."));
        if (name === "CustomField" || name === "CustomObject") {
            //Custom Field or Custom Object
            result = componentName.endsWith('__c') || componentName.endsWith('__mdt');
        }
        return result;
    }
    static getMemberNameFromFilepath(filepath, name) {
        var member;
        var splitFilepath = filepath.split(path.sep);
        var lastIndex = splitFilepath.length - 1;
        if (name === 'CustomField' || name === 'RecordType' || name === 'ListView' || name === 'ValidationRule' || name === 'WebLink' || name === 'CompactLayout' || name === 'BusinessProcess') {
            var objectName = splitFilepath[lastIndex - 2];
            var fieldName = splitFilepath[lastIndex].split('.')[0];
            member = objectName.concat('.' + fieldName);
        }
        else if (name === 'Dashboard' || name === 'Document' || name === 'EmailTemplate' || name === 'Report') {
            let baseName = "";
            switch (name) {
                case "Dashboard":
                    baseName = "dashboards";
                    break;
                case "Document":
                    baseName = "documents";
                    break;
                case "EmailTemplate":
                    baseName = "email";
                    break;
                case "Report":
                    baseName = "reports";
                    break;
            }
            let baseIndex = filepath.lastIndexOf(baseName) + baseName.length;
            var cmpPath = filepath.substring(baseIndex + 1); // add 1 to remove the path seperator
            cmpPath = cmpPath.substring(0, cmpPath.indexOf("."));
            member = cmpPath.replace(path.sep, "/");
        }
        else if (name === "QuickAction") {
            member = splitFilepath[lastIndex].replace('.quickAction-meta.xml', '');
        }
        else if (name === "CustomMetadata") {
            member = splitFilepath[lastIndex].replace(".md-meta.xml", '');
        }
        // Add exceptions for member names that need to be treated differently using else if
        else {
            member = splitFilepath[lastIndex].split('.')[0];
        }
        return member;
    }
    //Determine the name based on the filename, for example, ApexClass, CustomObject etc
    //NEED TO TAKE INTO CONSIDERATION OTHER TYPES AS WELL
    static getNameOfTypes(filename) {
        let name;
        if (filename.endsWith('.settings-meta.xml')) {
            name = 'AccountSettings';
        }
        else if (filename.endsWith('.actionLinkGroupTemplate-meta.xml')) {
            name = 'ActionLinkGroupTemplate';
        }
        else if (filename.endsWith('.cls-meta.xml')) {
            name = 'ApexClass';
        }
        else if (filename.endsWith('.component-meta.xml')) {
            name = 'ApexComponent';
        }
        else if (filename.endsWith('.page-meta.xml')) {
            name = 'ApexPage';
        }
        else if (filename.endsWith('.trigger-meta.xml')) {
            name = 'ApexTrigger';
        }
        else if (filename.endsWith('.approvalProcess-meta.xml')) {
            name = 'ApprovalProcess';
        }
        else if (filename.endsWith('.assignmentRules-meta.xml')) {
            name = 'AssignmentRules';
        }
        else if (filename.endsWith('.authProvider-meta.xml')) {
            name = 'AuthProvider';
        }
        else if (filename.endsWith('.app-meta.xml')) {
            name = 'CustomApplication';
        }
        else if (filename.endsWith('.customApplicationComponent-meta.xml')) {
            name = 'CustomApplicationComponent';
        }
        else if (filename.endsWith('.object-meta.xml')) {
            name = 'CustomObject';
        }
        else if (filename.endsWith('.md-meta.xml')) {
            name = 'CustomMetadata';
        }
        else if (filename.endsWith('.field-meta.xml')) {
            name = 'CustomField';
        }
        else if (filename.endsWith('.webLink-meta.xml')) {
            name = 'WebLink';
        }
        else if (filename.endsWith('.listView-meta.xml')) {
            name = 'ListView';
        }
        else if (filename.endsWith('.validationRule-meta.xml')) {
            name = 'ValidationRule';
        }
        else if (filename.endsWith('.tab-meta.xml')) {
            name = 'CustomTab';
        }
        else if (filename.endsWith('.dashboard-meta.xml') || filename.endsWith('.dashboardFolder-meta.xml')) {
            name = 'Dashboard';
        }
        else if (filename.endsWith('.document-meta.xml')) {
            name = 'Document';
        }
        else if (filename.endsWith('.email-meta.xml') || filename.endsWith('.emailFolder-meta.xml')) {
            name = 'EmailTemplate';
        }
        else if (filename.endsWith('.delegateGroup-meta.xml')) {
            name = 'DelegateGroup';
        }
        else if (filename.endsWith('.duplicateRule-meta.xml')) {
            name = 'DuplicateRule';
        }
        else if (filename.endsWith('.objectTranslation-meta.xml')) {
            name = 'CustomObjectTranslation';
        }
        else if (filename.endsWith('.recordType-meta.xml')) {
            name = 'RecordType';
        }
        else if (filename.endsWith('.layout-meta.xml')) {
            name = 'Layout';
        }
        else if (filename.endsWith('.quickAction-meta.xml')) {
            name = 'QuickAction';
        }
        else if (filename.endsWith('.report-meta.xml') || filename.endsWith('.reportFolder-meta.xml')) {
            name = 'Report';
        }
        else if (filename.endsWith('.resource-meta.xml')) {
            name = 'StaticResource';
        }
        else if (filename.endsWith('.translations-meta.xml')) {
            name = 'Translations';
        }
        else if (filename.endsWith('.role-meta.xml')) {
            name = 'Role';
        }
        else if (filename.endsWith('.queue-meta.xml')) {
            name = 'Queue';
        }
        else if (filename.endsWith('.group-meta.xml')) {
            name = 'Group';
        }
        else if (filename.endsWith('.profile-meta.xml')) {
            name = 'Profile';
        }
        else if (filename.endsWith('.permissionset-meta.xml')) {
            name = 'PermissionSet';
        }
        else if (filename.endsWith('.site-meta.xml')) {
            name = 'CustomSite';
        }
        return name;
    }
}
exports.default = MetadataFiles;
//# sourceMappingURL=metadataFiles.js.map